@@ -185,7 +185,7 @@ class Repository(object):
 			return self.listStash()
 
 
-	def pull(self, force: bool = False):
+	def pull(self, force: bool = False, pullSubmodules: bool = False):
 		if self.isDirty():
 			if not force:
 				raise DirtyRepository()
@@ -194,10 +194,27 @@ class Repository(object):
 
 		self.execute('git pull')
 
+		if pullSubmodules:
+			self.pullSubmodules(force=force)
 
-	def fetch(self):
+
+	def pullSubmodules(self, force: bool = False):
+		if force:
+			self.execcute('git submodule foreach git stash')
+
+		self.execute('git submodule foreach git pull')
+
+
+	def fetch(self, fetchSubmodules: bool = False):
 		self.execute('git fetch')
 
+		if fetchSubmodules:
+			self.fetchSubmodules()
+
+
+	def fetchSubmodules(self):
+		self.execute('git submodule foreach git fetch')
+
 
 	def reset(self):
 		self.execute('git reset --hard')
@@ -263,8 +280,8 @@ class Repository(object):
 
 		if autoAdd:
 			out, err = self.execute('git add --all')
-		if err:
-			return False
+			if err:
+				return False
 
 		cmd = f'git commit -m "{message}"'
 		out, err = self.execute(cmd)
@@ -395,3 +412,28 @@ class Remote(object):
 			return proc.stderr
 
 		return int(proc.stdout.strip())
+
+
+	def lsRemote(self, heads: bool = True, tags: bool = True):
+		"""
+		List references in a remote repository
+		types: tag for tags and head for branches
+		:return:
+		"""
+		option = f'{"-" if heads or tags else ""}' \
+		         f'{"h" if heads else ""}' \
+		         f'{"t" if tags else ""}'
+		proc = subprocess.run(f'git -C {str(self.repository.path)} ls-remote {option} {self.name}', shell=True, capture_output=True, text=True)
+
+		if not proc.stdout and proc.stderr:
+			raise Exception(proc.stderr)
+
+		out = dict()
+		for line in proc.stdout.strip().split('\n'):
+			commit,path = line.split('\t')
+			type = 'tag' if path.startswith('refs/tags/') else 'head'
+			out[path[10:] if type == 'tag' else path[11:]] = { 'ref': path ,
+												              'type': type,
+										                    'commit': commit }
+
+		return out
